% Rust Model
clear
addpath(genpath('functions'))

rng(12345)

K=90;J=1;

% 1. Data-generating parameters / Generate data
% 1(a). Utility patameter 
theta1_true= 5.0727;
theta2_true= -0.002293;
u1 = zeros(K,1);
u0 = theta1_true + theta2_true*(1:K)';
u_mat_true=[u0,u1];
% 1(b). Distribution is logistic 
% 1.(c) Known objects to the researcher (G,beta) 
beta=0.999;
p_i=[0.3919,0.5953,1-(0.3919+0.5953)]';
G0      = diag( p_i(1)*ones(K,1)) + diag( p_i(2)*ones(K-1,1),1) + diag( p_i(3)*ones(K-2,1),2);
G0(K,K) = 1; G0(K-1,K) =  p_i(2) +  p_i(3); 
G1=zeros(K,K);G1(:,1) =  p_i(1); G1(:,2) =  p_i(2); G1(:,3) =  p_i(3);
G=[G0;G1];
% 1. (d) Generate the Sufficient statistics (i.e. data) using the true CCP and a large "N"
[ccp_true, ev_mat,v_mat,Q] = solveDP_dyn_logit(u_mat_true,G,beta,100000);
n_large=10;
n_decision=n_large*ccp_true;

% 3. Researcher's modeling 
% 3. (a) Resercher pre-specifies "u_fixed" 
% researcher uses pre-specified values for theta1-theta3
% which happens to be the true values
D=2;S=D-(J+1);
Z_1_1toJp1   = [zeros(K,1), zeros(K,1)];
Z_0_1toJp1   = [ones(K,1) , (1:K)'    ];
Z_mat_1toJp1=[Z_1_1toJp1;Z_0_1toJp1];
theta_fixed=[theta1_true theta2_true ]';
u_fixed=Z_mat_1toJp1*theta_fixed;
% The parameter part
include_u=0;
Z_mat_Jp2toD=[];


% 3.(c) Chooises prior
prior_W_mu    =[0.5 0.5];
prior_W_s_comp=[0.4 0.6];
prior_L_mu    =[2.5 1.5];
prior_L_s_comp=[0   -6 ];
prior_S_mu    =[1   5  ];
prior_S_s_comp=[1   1  ];
prior_WEIGHT  =[ 0.5 0.5 ; prior_W_mu ; prior_W_s_comp ; 0.5  0.5  ; 0.5 0.5];
prior_LOCATION=[ 0   0   ; prior_L_mu ; prior_L_s_comp ; 0    0    ; 0   0  ];
prior_SCALE   =[ 1   1   ; prior_S_mu ; prior_S_s_comp ; 0.01 0.01 ; 3   3  ];

dir_hyp=10;
max_m=7;
Am_=0.05;
Amlogp_=5;
pi_m=exp(  -Am_.*(1:max_m).*(log(1:max_m).^Amlogp_)  );
pi_m/sum(pi_m) 


% ESTIMATION 
Npara_trans = (1:max_m)-1+J*(1:max_m)+(1:max_m)+1+S*include_u;
 
Nsim=10;                                         % Num of iterations per HMC 
Nmcmc_iter=100;500000;                               % Num of JDT outer-iterations 
m_sim=zeros(1,Nmcmc_iter);                       % Store m 
 
para_trans_sim=zeros(max_m-1+J*max_m+max_m+1+S*include_u,Nmcmc_iter);  % Store para_trans
 


% 3-1-2 Tune HMC para using the first data (to save time)
init_para_trans_specified=[];
parfor ind_m=1:max_m
smp_tuned_store{ind_m}=tune_hmc_given_m(ind_m,n_decision,init_para_trans_specified,prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
    G,beta,u_fixed,Z_mat_Jp2toD,include_u);
fprintf('JUST TUNED for #%d\n', ind_m);
end 
% 3-1-3 Initialize m and para_trans
m_sim(1) = 3;

accept=zeros(Nmcmc_iter,1);
tic;
for mcmc_iter=2:Nmcmc_iter

% 1. Run HMC given m for Nsim draws    
temp=run_hmc_given_m(m_sim(mcmc_iter-1),n_decision,Nsim,smp_tuned_store,init_para_trans_specified,...
    prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
    G,beta,u_fixed,Z_mat_Jp2toD,include_u)';

% 2. Run the block for m 
[m_sim(mcmc_iter),para_trans_sim(1:Npara_trans(m_sim(mcmc_iter)),mcmc_iter)]=run_jump_step(...
    temp(:,end),m_sim(mcmc_iter-1),...
    prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,Am_,Amlogp_,...
    n_decision,J,K,G,beta,S,include_u,theta_fixed,u_fixed,Z_mat_Jp2toD,max_m);

init_para_trans_specified=para_trans_sim(1:Npara_trans(m_sim(mcmc_iter)),mcmc_iter);

fprintf('JUST FINISHED ITERATION #%d\n', mcmc_iter);
 
end 
toc; 



data=[m_sim;para_trans_sim];
burnin=0.2*Nmcmc_iter;slice=1;
data=data(:,burnin+1:slice:end);