function[grad_A_omega, grad_a_omega,...
         grad_A_mu,    grad_a_mu,...
         grad_A_sigma, grad_a_sigma,...
         grad_A_u0,    grad_a_u0,...
         grad_A_ud,    grad_a_ud]...
=comp_grad_as_para(omega,sigma,v_mat,beta,G,D,F,E1,...
grad_Q_omega,grad_Q_mu,grad_Q_sigma,grad_Q_u0,grad_Q_ud)

K=size(v_mat,1);Nd=size(v_mat,2);J=Nd-1;m=size(omega,1);
v0=v_mat(:,1);G0=G(1:K,:);
%[D,F,E1]=comp_DF(v,mu,sigma,A,G,J,K,m);
% Create indicators
ind_mu    = reshape(kron(eye(m),ones(K*J,1)),K,J*m,m);
ind_sigma = reshape(reshape(kron(eye(m),ones(K,1)),K,m*m),K,m,m);
ind_u0    = repmat(eye(K),1,1,m);
ind_ud    = repmat(kron(eye(K),ones(1,J)),1,1,m);

% Convert sigma to a K by L by m array i.e. m KbyL matrices. 
% For each k=1:m, the KbyL matrix = ones(K,L)*1/sigma(k)
inv_sigma_3d_Jm = repmat(reshape(1./sigma,1,1,m),K,J*m); % for 2
inv_sigma_3d_m  = repmat(reshape(1./sigma,1,1,m),K,m  ); % for 3
inv_sigma_3d_K  = repmat(reshape(1./sigma,1,1,m),K,K  ); % for 4
inv_sigma_3d_JK = repmat(reshape(1./sigma,1,1,m),K,J*K); % for 5

% Derivatives of "A" wrt (a)mu,(b)sigma,and (c)ud have second parts that we define here
% (a) wrt mu
ind_row_mu=reshape( (repmat(1:m:1+(K-1)*m,      m,1)+[0:(m-1)]')',1,K*m  );
ind_col_mu=reshape( (repmat(1:m*m:1+(J-1)*m*m,m*m,1)+[0:m*m-1]')',1,J*m*m);
mat=kron(reshape(D(ind_row_mu,:),K,J*m),ones(1,m));
expD_mu=exp(reshape(mat(:,ind_col_mu),K,m*J,m));
part2_grad_A_mu=inv_sigma_3d_Jm.*ind_mu.*expD_mu;
% (b) wrt sigma
original = 1:m:1+(K-1)*m;
ind_col_sigma=reshape((repmat(original,m,1)+[0:m-1]')',1,K*m);
vec=sum(exp(D).*(E1./repmat(sigma,K,J)),2);
part2_grad_A_sigma=-reshape(kron(reshape(vec(ind_col_sigma),K,m),ones(1,m)),K,m,m).*ind_sigma;
% (c) wrt ud 
ind_row_ud=ind_row_mu;
ind_col_ud=reshape((repmat(1:m*K:1+(J-1)*m*K,K*m,1)+[0:K*m-1]')',1,J*m*K);
mat=kron(reshape(D(ind_row_ud,:),K,J*m),ones(1,K));
expD_ud=exp(reshape(mat(:,ind_col_ud),K,K*J,m));
part2_grad_A_ud=ind_ud.*inv_sigma_3d_JK.*expD_ud;
% Derivatives of "a" wrt (d)u0 and (e)sigma have second parts that we define here
% (d) wrt u0
part2_grad_a_u0=ind_u0.*inv_sigma_3d_K;
% (e) wrt sigma
part2_grad_a_sigma=-(inv_sigma_3d_m.^2).*ind_sigma.*repmat(v0,1,m,m);

if m>1
% 1. grad_A(a)_omega
grad_A_omega = comp_part1_grad_A_para_orign(m-1,grad_Q_omega,F,sigma,m,K,beta);
grad_a_omega = comp_part1_grad_small_a_para_orign(m-1,grad_Q_omega,grad_A_omega,G0,sigma,m,K,beta);
elseif m==1
grad_A_omega=[];
grad_a_omega=[];
end 
% 2. grad_A(a)_mu
grad_A_mu    = comp_part1_grad_A_para_orign(m*J,grad_Q_mu,F,sigma,m,K,beta)...
                   +part2_grad_A_mu;
grad_a_mu    = comp_part1_grad_small_a_para_orign(m*J,grad_Q_mu,grad_A_mu,G0,sigma,m,K,beta);
% 3. grad_A(a)_sigma
grad_A_sigma = comp_part1_grad_A_para_orign(m,grad_Q_sigma,F,sigma,m,K,beta)...
                   +part2_grad_A_sigma;
grad_a_sigma = comp_part1_grad_small_a_para_orign(m,grad_Q_sigma,grad_A_sigma,G0,sigma,m,K,beta)...
                   +part2_grad_a_sigma;
% 4. grad_A(a)_u0
grad_A_u0    = comp_part1_grad_A_para_orign(K,grad_Q_u0,F,sigma,m,K,beta);
grad_a_u0    = comp_part1_grad_small_a_para_orign(K,grad_Q_u0,grad_A_u0,G0,sigma,m,K,beta)...
                   +part2_grad_a_u0;
% 5. grad_A(a)_ud
grad_A_ud    = comp_part1_grad_A_para_orign(J*K,grad_Q_ud,F,sigma,m,K,beta)...
                   +part2_grad_A_ud;
grad_a_ud    = comp_part1_grad_small_a_para_orign(J*K,grad_Q_ud,grad_A_ud,G0,sigma,m,K,beta);
end 



