% Parameters    : para_trans (m)
% User-defined  : theta_hat (this can be ignored if include_u==1)
% Data          : n_decision (K by Nd)
% Known Objects : G, beta, u_fixed,Z_mat_Jp2toD, include_u
% Others        : Q_init 
function [loglik,grad_loglik]= comp_loglik_new(para_trans,m,n_decision,G,beta,u_fixed,Z_mat_Jp2toD,include_u,Q_init)
[K,Nd]=size(n_decision);J=Nd-1;S=size(Z_mat_Jp2toD,2);
% Unpack para_trans
% para_trans =[alpha;mu_vec;s;s_common;theta(J+2:D)] m-1+Jm+m+1+S by 1 if include_u==1
% para_trans =[alpha;mu_vec;s;s_common]              m-1+Jm+m+1   by 1 if include_u==0
Nalpha        = m-1;
Nmu_vec       = J*m;
Ns_comp       = m;
Ns_common     = 1;
Ntheta_Jp2toD = S;
N1            = Nalpha;
N2            = Nalpha+Nmu_vec;
N3            = Nalpha+Nmu_vec+Ns_comp;
N4            = Nalpha+Nmu_vec+Ns_comp+Ns_common;
N5            = Nalpha+Nmu_vec+Ns_comp+Ns_common+Ntheta_Jp2toD;
alpha         = para_trans(1:N1);
mu_vec        = para_trans(1+N1:N2);
s_comp        = para_trans(1+N2:N3);
s_common      = para_trans(1+N3:N4);
if include_u==1
theta_Jp2toD  = para_trans(1+N4:N5);
end 
mu=reshape(mu_vec',m,J)';
% Define u_mat = [u_0,u_1,...,u_J]
if include_u==1
u_para=Z_mat_Jp2toD*theta_Jp2toD;
u_mat=reshape(u_fixed+u_para,K,J+1);
elseif include_u==0
u_mat=reshape(u_fixed,K,J+1);
end 
ind=[J+1 (1:J)];
u_mat=u_mat(:,ind);

% Transform (alpha,mu_vec,s,s_common,theta(J+2:D)) 
% to (omega,mu_vec,sigma_comp,sigma_comp_common,theta(J+2:D))
% para =[omega;mu_vec;sigma_comp;sima_all;theta(J+2:D)] m+Jm+m+1+S by 1 if include_u==1
% para =[omega;mu_vec;sigma_comp;sima_all]              m+Jm+m+1   by 1 if include_u==0
para = trans_inv_var(para_trans,m,J,S,include_u);
Nomega        = m;
Nmu_vec       = J*m;
Nsigma        = m;
Nsigma_common = 1;
Ntheta_Jp2toD = S;
S1            = Nomega;
S2            = Nomega+Nmu_vec;
S3            = Nomega+Nmu_vec+Nsigma;
%S4           = Nomega+Nmu_vec+Nsigma+Nsigma_common;
%S5           = Nomega+Nmu_vec+Nsigma+Nsigma_common+Ntheta_Jp2toD;
omega         = para(1:S1);
sigma         = para(1+S2:S3);

[~,v_mat,~,~] = solveDP_evmix_NK(u_mat,G,mu,sigma,omega,beta,100,100000,Q_init);
% 2. Log-Likelihood
[ccp,ccp_mixcomp] = comp_ccp(v_mat,mu,sigma,omega);
loglik = sum( sum( n_decision.*log(ccp) ) );

% 3. Gradients of Log-Likelihood wrt parameters in original forms 
[A,a] = comp_as(v_mat,mu,sigma,omega);
if m>1
[grad_Q_omega,grad_Q_mu,grad_Q_sigma,grad_Q_u0,grad_Q_ud]...
=comp_ift_Q_new(omega,mu,sigma,a,A,ccp,ccp_mixcomp,v_mat,beta,G);

[grad_P_omega,grad_P_mu,grad_P_sigma,grad_P_u0,grad_P_ud]...
    =comp_grad_P_para(omega,mu,sigma,a,A,ccp_mixcomp,v_mat,beta,G,...
    grad_Q_omega,grad_Q_mu,grad_Q_sigma,grad_Q_u0,grad_Q_ud);
elseif m==1
[~,grad_Q_mu,grad_Q_sigma,grad_Q_u0,grad_Q_ud]...
=comp_ift_Q_new(omega,mu,sigma,a,A,ccp,ccp_mixcomp,v_mat,beta,G);  
[~,grad_P_mu,grad_P_sigma,grad_P_u0,grad_P_ud]...
    =comp_grad_P_para(omega,mu,sigma,a,A,ccp_mixcomp,v_mat,beta,G,...
    [],grad_Q_mu,grad_Q_sigma,grad_Q_u0,grad_Q_ud);
end 

% 3-1. grad_loglik_omega
if m>1
grad_loglik_omega=comp_grad_loglik_para_origin(m-1,grad_P_omega, n_decision,ccp,K,Nd);
elseif m==1
grad_loglik_omega=[];    
end
% 3-2. grad_loglik_mu
grad_loglik_mu   =comp_grad_loglik_para_origin(J*m,grad_P_mu,    n_decision,ccp,K,Nd);
ind_mu=reshape( (repmat(1:J:1+(m-1)*J,J,1) + [0:1:J-1]')',1,J*m);
grad_loglik_mu=grad_loglik_mu(ind_mu);
% 3-3. grad_loglik_sigma
grad_loglik_sigma=comp_grad_loglik_para_origin(m,  grad_P_sigma, n_decision,ccp,K,Nd);
% 3-4. grad_loglik_u0
grad_loglik_u0   =comp_grad_loglik_para_origin(K,  grad_P_u0,    n_decision,ccp,K,Nd);
% 3-5. grad_loglik_ud
grad_loglik_ud   =comp_grad_loglik_para_origin(K*J,grad_P_ud,    n_decision,ccp,K,Nd);
ind_ud=reshape( (repmat(1:J:1+(K-1)*J,J,1) + [0:1:J-1]')',1,J*K);
grad_loglik_ud=grad_loglik_ud(ind_ud);

% 4. Gradients of Log-Likelihood wrt parameters in transformed forms 
% 4-1. grad_loglike_alpha 
if m>1
max_alpha=max(alpha);
lambda = alpha - max_alpha;
grad_omega_alpha=( exp(-max_alpha) + sum(exp(lambda)) )^(-2)*(...
- exp(lambda'+lambda )...
+ exp(lambda'+lambda).*eye(m-1)...
+diag( ( exp(-max_alpha)+sum(exp(lambda))-exp(lambda') ).*exp(lambda')));
%sum(grad_omega_alpha,1); %should sum to ( exp(-max_alpha) + sum(exp(lambda)) )^(-2)*exp(lambda-max_alpha)
grad_loglik_alpha=grad_omega_alpha*grad_loglik_omega;
elseif m==1
grad_loglik_alpha=[];    
end 

% 4-2. grad_loglik_mu
% already given

% 4-3. grad_loglik_s(component-specific)
grad_loglik_s_comp=grad_loglik_sigma.*sigma;

% 4-4. grad_loglike_s(common)
grad_loglik_s_common=sum(grad_loglik_s_comp);

% 4-5. grad_loglik_theta
if include_u==1
grad_loglik_u=[grad_loglik_u0,reshape(grad_loglik_ud,K,J)] ;
Z_mat_Jp2toD_mod=[Z_mat_Jp2toD(K*J+1:end,:);Z_mat_Jp2toD(1:K*J,:)];
grad_loglik_theta_Jp2toD=sum(reshape(sum(repmat(grad_loglik_u,1,S).*reshape(Z_mat_Jp2toD_mod,K,S*Nd),1)',Nd,S),1)';
    if m>1
    grad_loglik=[grad_loglik_alpha;  grad_loglik_mu;
                grad_loglik_s_comp;  grad_loglik_s_common;
                grad_loglik_theta_Jp2toD];
    elseif m==1
    grad_loglik=[                    grad_loglik_mu;
                grad_loglik_s_comp;  grad_loglik_s_common;
                grad_loglik_theta_Jp2toD];    
    end 
elseif include_u==0
    if m>1
    grad_loglik=[grad_loglik_alpha;    grad_loglik_mu;
                grad_loglik_s_comp;    grad_loglik_s_common];  
    elseif m==1
    grad_loglik=[                      grad_loglik_mu;
                grad_loglik_s_comp;    grad_loglik_s_common];          
    end 
end 



   
end