% Conditional posterior of chi_{m+1} given the value of chi_{1:m} fixed
% i.e. Just evaluate posterior of chi_{1:m+1} but make a matlab function
% in terms of chi_{m+1} 
%
% 1. Def
% chi_{1:m}   = para_trans_unnor 
% chi_{m+1}   = para_trans_unnor_mp1 
% chi_{1:m+1} = para_trans_unnor_new 
%
% 2. Inputs 
% chi_{1:m}    = (gamma_{1:m}, mu_{1:J,1:m}, s_comp_{1:m}, s_common, theta's) 
% chi_{m+1}    = (gamma_{m+1}, mu_{1:J,m+1}, s_comp_{m+1} ), (J+2) by 1 
% chi_{1:m+1}  = (gamma_{1:m+1}, mu_{1:J,1:m+1}, s_comp_{1:m+1}, s_common,  theta's) 
%
% 3. Outputs 
% logpost     = MINUS of log posteior eval at para_trans_unnor_new (theta_{m+1}) 1 by 1 
% grad        = Der. of MINUS of log posterior wrt para_trans_unnor_mp1, (J+2) by 1 


function [logpost,grad]= comp_logpost_mp1(para_trans_unnor_mp1,para_trans_unnor,m,n_decision,...
    prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
    G,beta,u_fixed,Z_mat_Jp2toD,include_u,Q_init)
%global paraval
%paraval = para_trans_unnor_mp1;


Nd=size(n_decision,2);J=Nd-1;S=size(Z_mat_Jp2toD,2);

% 1. chi_{1:m}
Ngamma          = m;
Nmu_vec         = J*m;
Ns_comp         = m;
Ns_common       = 1;
Ntheta_Jp2toD   = S;
N1              = Ngamma;
N2              = Ngamma+Nmu_vec;
N3              = Ngamma+Nmu_vec+Ns_comp;
N4              = Ngamma+Nmu_vec+Ns_comp+Ns_common;
N5              = Ngamma+Nmu_vec+Ns_comp+Ns_common+Ntheta_Jp2toD;
gamma           = para_trans_unnor(1:N1);
mu_vec          = para_trans_unnor(1+N1:N2);
s_comp          = para_trans_unnor(1+N2:N3);
s_common        = para_trans_unnor(1+N3:N4);
if include_u==1
theta_Jp2toD    = para_trans_unnor(1+N4:N5);
end 
mu=reshape(mu_vec',m,J)';

% 2. chi_{m+1} = (gamma_{m+1}, mu_{1:J,m+1}, s_comp_{m+1} )
gamma_mp1  = para_trans_unnor_mp1(1);
mu_vec_mp1 = para_trans_unnor_mp1(2:1+J);
s_comp_mp1 = para_trans_unnor_mp1(end);

% 3. chi_{1:m+1} 
gamma_new   = [gamma;gamma_mp1];
mu_new      = [mu,mu_vec_mp1];mu_vec_new=reshape(mu_new',(m+1)*J,1);
s_comp_new  = [s_comp;s_comp_mp1];
if include_u==1
para_trans_unnor_new = [gamma_new;mu_vec_new;s_comp_new;s_common; theta_Jp2toD ];
else
para_trans_unnor_new = [gamma_new;mu_vec_new;s_comp_new;s_common ];    
end 

[logpost_mp1,grad_logpost_mp1] = comp_logpost_unnor_new(para_trans_unnor_new,m+1,n_decision,...
    prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
    G,beta,u_fixed,Z_mat_Jp2toD,include_u,Q_init);
logpost=-logpost_mp1;

% We only need derivatives for the (m+1)'s part of the parameter 
Ncomp   = m+1;
Ngamma  = Ncomp;
Nmu_vec = J*Ncomp;
Ns_comp = Ncomp;
N1      = Ngamma;
N2      = Ngamma+Nmu_vec;
N3      = Ngamma+Nmu_vec+Ns_comp;
grad_gamma  = grad_logpost_mp1(1:N1);
grad_mu_vec = grad_logpost_mp1(1+N1:N2);
grad_s_comp = grad_logpost_mp1(1+N2:N3);
grad_mu     = reshape(grad_mu_vec',Ncomp,J)';

grad=-[grad_gamma(end);grad_mu(:,end);grad_s_comp(end)];



end 