% Inputs:
% data: n_decision (=the new data in a given JDT iteration)
% smp_tuned_store = an array of tuned  smp information 
% init_para_trans_specified = the initial value specifed by the jump step for m 

% 1. Set initial value for HMC 
% 2. Use (1) the tuned parameters and (2) 'init_para_trans' to define 'smp' 
% 3. Run HMC with the smp 


function para_trans_sim=run_hmc_given_m(m,n_decision,Nsim,smp_tuned_store,init_para_trans_specified,...
    prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
    G,beta,u_fixed,Z_mat_Jp2toD,include_u)

[K,Nd]=size(n_decision);J=Nd-1;S=size(Z_mat_Jp2toD,2);

% Set initial value for HMC
if isempty(init_para_trans_specified)  
    
% 1-1. Set initial value for MAP estimation  
    if include_u==0
        init_para_trans = trans_var( [repmat(1/m,m,1);zeros(J*m,1);ones(m,1);1           ],m,J,S,include_u )+0.01*randn(m-1+J*m+m+1,1);    
    elseif include_u==1
        init_para_trans = trans_var( [repmat(1/m,m,1);zeros(J*m,1);ones(m,1);1;zeros(S,1)],m,J,S,include_u )+0.01*randn(m-1+J*m+m+1+S,1);  
    end 
% 1-2. Compute MAP 
    Q_init=zeros(K,1);
    log_post = @(para_trans)comp_logpost_new(para_trans,m,n_decision,prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
        G,beta,u_fixed,Z_mat_Jp2toD,include_u,Q_init);
    smp = hmcSampler(log_post,init_para_trans,'NumSteps',10,'StepSize',0.5,'CheckGradient',false);
    [para_trans_map,~] = estimateMAP(smp,'VerbosityLevel',1); 
    init_para_trans=para_trans_map;

    
else
    init_para_trans=init_para_trans_specified;
end

para_map = trans_inv_var(init_para_trans,m,J,S,include_u);
Nomega           = m;
Nmu_vec          = J*m;
Nsigma           = m;
Nsigma_all       = 1;
Ntheta_Jp2toD    = S;
S1               = Nomega;
S2               = Nomega+Nmu_vec;
S3               = Nomega+Nmu_vec+Nsigma;
S4               = Nomega+Nmu_vec+Nsigma+Nsigma_all;
S5               = Nomega+Nmu_vec+Nsigma+Nsigma_all+Ntheta_Jp2toD;
omega_map        = para_map(1:S1);
mu_vec_map       = para_map(1+S1:S2);
sigma_map        = para_map(1+S2:S3);
sigma_common_map = para_map(1+S3:S4);
if include_u==1
theta_Jp2toD_map = para_map(1+S4:S5);
end 

% 1-3. Using the MAP/initial value, specify the initial value for Emax 
if include_u==1
u_para=Z_mat_Jp2toD*theta_Jp2toD_map;
u_mat=reshape(u_fixed+u_para,K,J+1);
elseif include_u==0
u_mat=reshape(u_fixed,K,J+1);
end
% u_mat=[u_0,u_1,...u_J]
ind=[J+1 (1:J)];
u_mat=u_mat(:,ind);
mu_map=reshape(mu_vec_map',m,J)';

[Q_init,~,~,~] = solveDP_evmix_NK(u_mat,G,mu_map,sigma_map,omega_map,beta,50,100000,zeros(K,1)) ;
log_post = @(para_trans)comp_logpost_new(para_trans,m,n_decision,prior_WEIGHT,prior_LOCATION,prior_SCALE,dir_hyp,...
    G,beta,u_fixed,Z_mat_Jp2toD,include_u,Q_init);


% 2. Use (1) the tuned parameters and (2) 'init_para_trans' to define 'smp' 
smp_tuned=smp_tuned_store{m};
smp = hmcSampler(log_post,init_para_trans,...
    'StepSize',smp_tuned.StepSize,...
    'NumSteps',smp_tuned.NumSteps,...
    'MassVector',smp_tuned.MassVector,...
    'JitterMethod',smp_tuned.JitterMethod,...
    'StepSizeTuningMethod',smp_tuned.StepSizeTuningMethod,...
    'MassVectorTuningMethod',smp_tuned.MassVectorTuningMethod,...
    'CheckGradient',false);



% 3  Run HMC
NumChains = 1; % 1 chain for now 
chains = cell(NumChains,1);
Burnin = 0;
%tic;
for c = 1:NumChains
    chains{c} = drawSamples(smp,'Start',init_para_trans, ...
        'Burnin',Burnin,'NumSamples',Nsim, ...
        'VerbosityLevel',2,'NumPrint',300);
end
%toc;
para_trans_sim= chains{1};



end 